//############################################################################//
//Project Frontier                                                            //
//ZRG lander header                                                           //
//Copyright 2006 Frontier team                                                //
//############################################################################//
#define STRICT
#define ORBITER_MODULE
#define LOADBMP(id)(LoadBitmap(g_Param.hDLL,MAKEINTRESOURCE(id)))


#include "orbitersdk.h"
#include "stdio.h"
#include <string>
#include <sstream> 
using namespace std;
#include "resource.h"
#include "OrbiterSoundSDK3.h"
#define Welcometomare

typedef struct {          // used for textout
 HINSTANCE hDLL;
 HFONT hFont[1];
 HPEN hPen[2];
 HBRUSH hBrush[1];
}GDIParams;

///////
////////
////////REASSIGN HOLD ALT-(NO HOVER ENGINES)

//VC buttons
#define MFD1               1


#define KROT               20
#define HORLVL             21
#define PROGRD             22
#define RETROGRD           23
#define NMLP               24
#define NMLM               25
#define HDAT               26 //retro 
#define UNDOCK             27
#define MAIN               28
#define GEARUP             29
const double LANDERZRG_FUELMASS=  14000.0;
const double LANDERZRG_ISP=       2.6e4;
const double LANDERZRG_MAXMAINTH= 4e5; //5.5e5;
const double LANDERZRG_MAXRETROTH=1e5;
const double LANDERZRG_MAXRCSTH=  2e4;
const double LANDERZRG_MAXRCSTH2= 2e4;// for roll

const double GEAR_OPERATING_SPEED=0.06;// had to slow down the gear to keep from jumping off the ground

const int nsurf=60;
GDIParams g_Param;

//############################################################################//
//############################################################################//
// ==============================================================
// Airfoil coefficient functions
// Return lift,moment and zero-lift drag coefficients as a
// function of angle of attack(alpha or beta)
// ==============================================================
// 1. vertical lift component(wings and body)
void VLiftCoeff(double aoa,double M,double Re,double *cl,double *cm,double *cd)
{ 
 int i;
 const int nabsc=9;
 static const double AOA[nabsc]={-180*RAD,-60*RAD,-30*RAD,-2*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
 static const double CL[nabsc] ={       0,     0,  -0.4,    0,   0.7,    1,  0.8,    0,     0};
 static const double CM[nabsc] ={       0,     0, 0.014,0.004,-0.0074,-0.094,-0.012,    0,     0};
 for(i=0; i < nabsc-1 && AOA[i+1] < aoa; i++);
 double f =(aoa-AOA[i]) /(AOA[i+1]-AOA[i]);
 *cl=CL[i] +(CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
 *cm=CM[i] +(CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
 double saoa=sin(aoa);
 double pd=0.015 + 0.4*saoa*saoa;  // profile drag
 *cd=pd + oapiGetInducedDrag(*cl,1.5,0.7) + oapiGetWaveDrag(M,0.75,1.0,1.1,0.04);
 // profile drag +(lift-)induced drag + transonic/supersonic wave(compressibility) drag
}
// 2. horizontal lift component(vertical stabilisers and body)
void HLiftCoeff(double beta,double M,double Re,double *cl,double *cm,double *cd)
{    
 int i;
 const int nabsc=8;
 static const double BETA[nabsc]={-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
 static const double CL[nabsc]  ={       0,   +0.3,     0,  -0.3, +0.3,    0,  -0.3,     0};
 for(i=0; i < nabsc-1 && BETA[i+1] < beta; i++);
 *cl=CL[i] +(CL[i+1]-CL[i]) *(beta-BETA[i]) /(BETA[i+1]-BETA[i]);
 *cm=0.0;
 *cd=0.015 + oapiGetInducedDrag(*cl,1.5,0.6) + oapiGetWaveDrag(M,0.75,1.0,1.1,0.04);
}
//############################################################################//
//############################################################################//

class LANDERZRG: public VESSEL2 {
public:
 LANDERZRG(OBJHANDLE hVessel,int flightmodel):VESSEL2(hVessel,flightmodel){    
  DefineAnimations();
  //SetGearParameters(gear_proc);
 }
    
 bool clbkLoadVC(int id);
 void InitVC(int vc);
 void VerticalLaunch();
 void RedrawNavMode();
 void DefineAnimations(void); // Useful for animation
 void RevertGear(void); // Operates the gear only
 void clbkLoadStateEx(FILEHANDLE scn,void *vs);//loads the status from the scenario file
 void clbkSaveState(FILEHANDLE scn);// for saving status to scenario file
 //void SetGearParameters(double state);//used for gear up vs. gear down landing points
 void clbkSetClassCaps(FILEHANDLE cfg);
 void clbkPostStep(double simt,double simdt,double mjd); // Animation
 void clbkPreStep (double simt,double simdt,double mjd); //Sound
 void clbkPostCreation (void);//for sound
 int  clbkConsumeBufferedKey(DWORD key,bool down,char *kstate); // Used to control events such as animation
 bool clbkLoadGenericCockpit();
 bool clbkVCMouseEvent(int id,int event,VECTOR3 &p);
 void ReleaseSurfaces();
 void clbkFocusChanged(bool getfocus,OBJHANDLE hNewVessel,OBJHANDLE hOldVessel);
 void clbkNavMode(int mode,bool active);
 void RedrawPanel_MFDButton(SURFHANDLE surf,int mfd,int side);

 private:
 int FM;
 UINT anim_gear;
 double gear_proc;

 enum GearStatus { GEAR_UP,GEAR_DOWN,GEAR_RAISING,GEAR_LOWERING } gear_status;

 int c;
 double playZland2,playZland3,playZland4;// added to play sound once
 double playGLand2,playGLand3,playGLand4;// added to play sound once
 SURFHANDLE srf[nsurf];
 MESHHANDLE exmesh,exmesh_tpl;  // local external mesh and global template
 MESHHANDLE vcmesh,vcmesh_tpl;
 MESHHANDLE ZRGpanel;



 VECTOR3 RelPos;          //
 VECTOR3 MinDist ;        // distance to trigger sound
 VECTOR3 RelPos1;
 VECTOR3 RelPos2;
 VECTOR3 RelPos3;         
 VECTOR3 MinDist1;
 VECTOR3 MinDist2; 
 VECTOR3 MinDist3; 

 OBJHANDLE hvessel;       
 double krotation;
 double horizon;
 double prograde;
 double retgrade;
 double orbitn;
 double orbitan;
 double holdalt;
 double gearpos;
 double tgl;

 SURFHANDLE tex2 ;// mesh ,texture#   krot
 SURFHANDLE tex3 ;// mesh ,texture#  hlvl
 SURFHANDLE tex4;// mesh ,texture#  prograde
 SURFHANDLE tex5 ;// mesh ,texture#  retrograde
 SURFHANDLE tex6 ;// mesh ,texture#  orbit norm
 SURFHANDLE tex7 ;// mesh ,texture#  orbit antinorm
 SURFHANDLE tex8 ;// mesh ,texture#  hold alt
 SURFHANDLE tex9 ;// mesh ,texture#  panel
 SURFHANDLE tex10 ;// mesh ,texture#  undock
 SURFHANDLE tex11 ;// mesh ,texture#  main off
 SURFHANDLE tex12 ;// mesh ,texture# gear
 SURFHANDLE tex13 ;// mesh ,texture# gearstat
 SURFHANDLE tex14 ;// mesh ,texture# FUELstat
 SURFHANDLE tex15 ;// mesh ,texture# l mfd stat
 SURFHANDLE tex16 ;// mesh ,texture# r mfd stat
 SURFHANDLE tex17 ;// mesh ,texture# mfd all

 // SURFHANDLE mfd1srf;//not used yet
 PROPELLANT_HANDLE hpr;


 int osnd;
 int cscn,scns,cvid;
};
