#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK3.h"

const double ZENI_FUELMASS = 50;
const double VAC_ZENI_ISP = 6.25e8;
const double NML_ZENI_ISP = 1e1;
const double AS_MAXRCSTH = 1e3;
const double P_NML = 1e1;




class AS: public VESSEL2 {
public:
	AS (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep (double simt, double simdt, double mjd);
	void clbkPostCreation(void);
	int Spaceman;
	
private:
MESHHANDLE exmesh_tpl;
double Play_Sound1;


    VECTOR3 RelPos1;         
    VECTOR3 MinDist1;
     OBJHANDLE hbase;
};


void AS::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_retro, th_rcs[14], th_group[4];

	SetSize (3);
	SetEmptyMass (500);
	SetCW (0.09, 1.0, 1.8, 1.8);
	SetReentryTexture (NULL);
	SetWingAspect (0.7);
	SetWingEffectiveness (2.5);
	SetCrossSections (_V(10.5,15.0,5.8));
	SetRotDrag (_V(0.6,0.6,0.35));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (5e-5);
		SetBankMomentScale (5e-5);
	}
	SetPMI (_V(131.19,156.2,31.43));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.0,0));
	Play_Sound1 = 0 ;
	SetSurfaceFrictionCoeff (0.26, 0.6);
	SetTouchdownPoints (_V(0,-1,2), _V(-1,-1,-2), _V(1,-1,-2));


	// propellant resources
	PROPELLANT_HANDLE hpr;
	hpr = CreatePropellantResource (ZENI_FUELMASS);
	

	
	th_main = CreateThruster (_V(0,0,-.5), _V(0,0,1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	th_retro = CreateThruster (_V(0,0,.5), _V(0,0,-1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);
	
	

	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V(1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V(1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0,1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	// visual specs defined in config
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new AS (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (AS*)vessel;
}


void AS::clbkPostCreation (void)
{
	Spaceman=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(Spaceman,1,"Sound\\Rho1\\JTEMU\\Rescue.wav",DEFAULT);
	SoundOptionOnOff3(Spaceman,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff3(Spaceman,DISPLAYTIMER,FALSE);
	SoundOptionOnOff3(Spaceman,PLAYRADIOATC,FALSE);
	SoundOptionOnOff3(Spaceman,PLAYGPWS,FALSE);
	
}


void AS::clbkPreStep (double simt, double simdt, double mjd) 
{
 ///////////////////////////////////////////////////////////////////
       // position relative to object
      VESSEL *v = oapiGetFocusInterface();// to determine which vehicle has focus  
      MinDist1 =  _V(10,10,10); //Distance to trigger sound
      hbase=oapiGetVesselByName("Debris-4");// name of vessel to detect distance (shows in F3 list)
	     if (oapiIsVessel(hbase) == true) { // check if vessel exist
		  v->GetRelativePos (hbase, RelPos1);//crashes if hvessel isnt in the senario
	  		                                     //
// change numbers to positive
       if (RelPos1.x < 0.0){
 	RelPos1.x = -(RelPos1.x) ;}
       if (RelPos1.y < 0.0){
    RelPos1.y = -(RelPos1.y) ;}
       if (RelPos1.z < 0.0){ 
    RelPos1.z = -(RelPos1.z) ;} 
   






     if (MinDist1.x > RelPos1.x && MinDist1.y > RelPos1.y && MinDist1.z > RelPos1.z){

		 if (Play_Sound1 < 1){ // play once !
    
			 PlayVesselRadioExclusiveWave3(Spaceman,1,255); 
			 Play_Sound1 = 1;


		 }// play once
	 }

	
		 } }