
#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK3.h"
const double ZENI_FUELMASS = 39000.0;
const double VAC_ZENI_ISP = 6.25e8;
const double NML_ZENI_ISP = 1e1;
const double ZENI_MAXMAINTH = 5.5e6;
const double GULL_MAXRCSTH = 1.32e6;
const double P_NML = 50e1;
const double HATCH_OPERATING_SPEED = 0.1;



class Gull: public VESSEL2 {
	
public:
	Gull (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {
	
	HATCHA_proc = 0.0;
	HATCHA_status = HATCHA_UP;
	HATCHB_proc = 0.0;
	HATCHB_status = HATCHB_UP;
	HATCHC_proc = 0.0;
	HATCHC_status = HATCHC_UP;
	HATCH_proc = 0.0;
	HATCH_status = HATCH_UP;
	DefineAnimations ();
	}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep (double simt, double simdt, double mjd);
	void clbkPostCreation(void);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);//loads the status from the scenario file
	void clbkSaveState (FILEHANDLE scn);// for saving status to scenario file
	void clbkPostStep (double simt, double simdt, double mjd); // Animation
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate); // Used to control events such as animation
	int Gullwind;
	enum HATCHStatus { HATCH_UP, HATCH_DOWN, HATCH_RAISING, HATCH_LOWERING } HATCH_status;
	enum HATCHAStatus { HATCHA_UP, HATCHA_DOWN, HATCHA_RAISING, HATCHA_LOWERING } HATCHA_status;
	enum HATCHBStatus { HATCHB_UP, HATCHB_DOWN, HATCHB_RAISING, HATCHB_LOWERING } HATCHB_status;
	enum HATCHCStatus { HATCHC_UP, HATCHC_DOWN, HATCHC_RAISING, HATCHC_LOWERING } HATCHC_status;
	void DefineAnimations (void); // Useful for animation
	void RevertHATCHA (void);
	void RevertHATCHB (void);
	void RevertHATCHC (void);
	void RevertHATCH (void);
		
private:
	
	UINT anim_HATCHA;
	double HATCHA_proc;
	UINT anim_HATCHB;
	double HATCHB_proc;
	UINT anim_HATCHC;
	double HATCHC_proc;
	UINT anim_HATCH;
	double HATCH_proc;
	MESHHANDLE exmesh_tpl;
	double Play_Sound1;
	
    VECTOR3 RelPos1;         
    VECTOR3 MinDist1;
     OBJHANDLE hbase;
   
};
	
void Gull::DefineAnimations (void)
{
	static UINT HATCHGrp[1] = {44};
	static MGROUP_TRANSLATE HATCH (0, HATCHGrp, 1, _V(0,0.3,0));
	
	static UINT LOWHATCHGrp [1] = {44};
	static MGROUP_TRANSLATE LOWHATCH (0, LOWHATCHGrp, 1, _V(0,0,7));
	
	anim_HATCH = CreateAnimation (0);
	AddAnimationComponent (anim_HATCH, 0, 0.5, &HATCH);
	AddAnimationComponent (anim_HATCH, 0.5, 1, &LOWHATCH);

	static UINT HATCHAGrp [2] = {20,47};
	static MGROUP_ROTATE HATCHA (0, HATCHAGrp, 2, _V(6.629,3.716,7.291),_V(-0.334,0.666,0), (float)(-170*RAD));
	
	anim_HATCHA = CreateAnimation (0);
	AddAnimationComponent (anim_HATCHA, 0, 1, &HATCHA);
	
	static UINT HATCHBGrp [2] = {19,46};
	static MGROUP_ROTATE HATCHB (0, HATCHBGrp, 2, _V(-6.629,3.716,7.291),_V(0.333,0.667,0), (float)(170*RAD));
	
	anim_HATCHB = CreateAnimation (0);
	AddAnimationComponent (anim_HATCHB, 0, 1, &HATCHB);
	
	static UINT HATCHCGrp [2] = {18,45};
	static MGROUP_ROTATE HATCHC (0, HATCHCGrp, 2, _V(0,-7.645,7.291),_V(1,0,0), (float)(-125*RAD));
	
	anim_HATCHC = CreateAnimation (0);
	AddAnimationComponent (anim_HATCHC, 0, 1, &HATCHC);

}

void Gull::RevertHATCHA (void)
{
	HATCHA_status = ((HATCHA_status == HATCHA_UP || HATCHA_status == HATCHA_RAISING) ?
		HATCHA_LOWERING : HATCHA_RAISING);
}
void Gull::RevertHATCHB (void)
{
	HATCHB_status = ((HATCHB_status == HATCHB_UP || HATCHB_status == HATCHB_RAISING) ?
		HATCHB_LOWERING : HATCHB_RAISING);
}
void Gull::RevertHATCHC (void)
{
	HATCHC_status = ((HATCHC_status == HATCHC_UP || HATCHC_status == HATCHC_RAISING) ?
		HATCHC_LOWERING : HATCHC_RAISING);
}

void Gull::RevertHATCH (void)
{
	HATCH_status = ((HATCH_status == HATCH_UP || HATCH_status == HATCH_RAISING) ?
		HATCH_LOWERING : HATCH_RAISING);
};


void Gull::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_rcs[14], th_group[4];

	SetSize (96);
	SetEmptyMass (400000);
	SetCW (0.09, 1.0, 1.8, 1.8);
	SetReentryTexture (NULL);
	SetWingAspect (0.7);
	SetWingEffectiveness (2.5);
	SetCrossSections (_V(1800,2500,370));
	SetRotDrag (_V(0.6,0.6,0.35));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (5e-5);
		SetBankMomentScale (5e-5);
	}
	SetPMI (_V(1654,1674,67));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-4.6,0.336,73));
	Play_Sound1 = 0;
	
    EnableTransponder (true);

	// ******************** NAV radios **************************

	InitNavRadios (2);

	
	SetTouchdownPoints (_V(0,16.8,-85.724), _V(14,-8,-85.724), _V(-14,-8,-85.724));

	// propellant resources
	PROPELLANT_HANDLE hpr;
	hpr = CreatePropellantResource (ZENI_FUELMASS);
	

	
	th_main = CreateThruster (_V(0,0,-20.5), _V(0,0,1), ZENI_MAXMAINTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	AddExhaust (th_main, 30, 2.5, _V(0,15.833,-82), _V(0,0,-1));
    AddExhaust (th_main, 30, 2.5, _V(-13.681,-7.912,-82), _V(0,0,-1));
    AddExhaust (th_main, 30, 2.5, _V(13.846,-7.747,-82), _V(0,0,-1));

	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0,1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0,1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0,1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0,1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V(1,0,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V(1,0,0), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0,1), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), GULL_MAXRCSTH, hpr, VAC_ZENI_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	// visual specs
	SetMeshVisibilityMode (AddMesh (exmesh_tpl = oapiLoadMeshGlobal ("Gull")), MESHVIS_ALWAYS);

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Gull (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (Gull*)vessel;
}
void Gull::clbkPostCreation (void)
{
	Gullwind=ConnectToOrbiterSoundDLL3(GetHandle());

	
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\Scout\\ZENI.wav",	REPLACE_MAIN_THRUST);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB1.wav",	REPLACE_COCKPIT_AMBIENCE_1);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB2.wav",	REPLACE_COCKPIT_AMBIENCE_2);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB3.wav",	REPLACE_COCKPIT_AMBIENCE_3);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB4.wav",	REPLACE_COCKPIT_AMBIENCE_4);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB7.wav",	REPLACE_COCKPIT_AMBIENCE_7);
	ReplaceStockSound3(Gullwind,"Sound\\Rho1\\landerZRG\\AMB8.wav",	REPLACE_COCKPIT_AMBIENCE_8);
	SoundOptionOnOff3(Gullwind,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff3(Gullwind,DISPLAYTIMER,FALSE);
	SoundOptionOnOff3(Gullwind,PLAYRADIOATC,FALSE);
	SoundOptionOnOff3(Gullwind,PLAYGPWS,FALSE);
	RequestLoadVesselWave3(Gullwind,1,"Sound\\Rho1\\Gullwind\\Visit.wav",DEFAULT);
}

void Gull::clbkPostStep (double simt, double simdt, double mjd)
{
	// Animate HATCH
	
if (HATCHA_status >= HATCHA_RAISING) {
		double da = simdt * HATCH_OPERATING_SPEED;
		if (HATCHA_status == HATCHA_RAISING) {
			if (HATCHA_proc > 0.0) HATCHA_proc = max (0.0, HATCHA_proc-da);
			else                HATCHA_status = HATCHA_UP;
		} else {
			if (HATCHA_proc < 1.0) HATCHA_proc = min (1.0, HATCHA_proc+da);
			else                HATCHA_status = HATCHA_DOWN;
		}
		SetAnimation (anim_HATCHA, HATCHA_proc);
}
if (HATCH_status >= HATCH_RAISING) {
		double da = simdt * HATCH_OPERATING_SPEED;
		if (HATCH_status == HATCH_RAISING) {
			if (HATCH_proc > 0.0) HATCH_proc = max (0.0, HATCH_proc-da);
			else                HATCH_status = HATCH_UP;
		} else {
			if (HATCH_proc < 1.0) HATCH_proc = min (1.0, HATCH_proc+da);
			else                HATCH_status = HATCH_DOWN;
		}
		SetAnimation (anim_HATCH, HATCH_proc);
}
if (HATCHB_status >= HATCHB_RAISING) {
		double da = simdt * HATCH_OPERATING_SPEED;
		if (HATCHB_status == HATCHB_RAISING) {
			if (HATCHB_proc > 0.0) HATCHB_proc = max (0.0, HATCHB_proc-da);
			else                HATCHB_status = HATCHB_UP;
		} else {
			if (HATCHB_proc < 1.0) HATCHB_proc = min (1.0, HATCHB_proc+da);
			else                HATCHB_status = HATCHB_DOWN;
		}
		SetAnimation (anim_HATCHB, HATCHB_proc);
}
if (HATCHC_status >= HATCHC_RAISING) {
		double da = simdt * HATCH_OPERATING_SPEED;
		if (HATCHC_status == HATCHC_RAISING) {
			if (HATCHC_proc > 0.0) HATCHC_proc = max (0.0, HATCHC_proc-da);
			else                HATCHC_status = HATCHC_UP;
		} else {
			if (HATCHC_proc < 1.0) HATCHC_proc = min (1.0, HATCHC_proc+da);
			else                HATCHC_status = HATCHC_DOWN;
		}
		SetAnimation (anim_HATCHC, HATCHC_proc);
}
}
int Gull::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events
		switch (key) {
			
		case OAPI_KEY_1: // deploy/retract HATCHA
			RevertHATCHA();
			return 1;
				
		case OAPI_KEY_G: // deploy/retract HATCHA
			RevertHATCH();
			return 1;
		case OAPI_KEY_2: // deploy/retract HATCHB
			RevertHATCHB();
			return 1;
		case OAPI_KEY_3: // deploy/retract HATCHC
			RevertHATCHC();
			return 1;
		}
		   	
    return 0;
	}
			
		
//Load from scn file
void Gull::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "HATCH", 6)) {
			sscanf (line+6, "%d%lf", &HATCH_status, &HATCH_proc);
			} else if (!strnicmp (line, "HATCH_A", 6)) {
			sscanf (line+6, "%d%lf", &HATCHA_status, &HATCHA_proc);
			} else if (!strnicmp (line, "HATCH_B", 6)) {
			sscanf (line+6, "%d%lf", &HATCHB_status, &HATCHB_proc);
			} else if (!strnicmp (line, "HATCH_C", 6)) {
			sscanf (line+6, "%d%lf", &HATCHC_status, &HATCHC_proc);
			} else {
			ParseScenarioLineEx (line, vs);
		}
	}
	
	SetAnimation (anim_HATCH, HATCH_proc);
	SetAnimation (anim_HATCHA, HATCHA_proc);
	SetAnimation (anim_HATCHB, HATCHB_proc);
	SetAnimation (anim_HATCHC, HATCHC_proc);
	}
////save to scn file
void Gull::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];
	VESSEL2::clbkSaveState (scn);
	SaveDefaultState (scn);
	

		sprintf (cbuf, "%d %0.4f", HATCH_status, HATCH_proc);
		oapiWriteScenario_string (scn, "HATCH", cbuf);
		sprintf (cbuf, "%d %0.4f", HATCHA_status, HATCHA_proc);
		oapiWriteScenario_string (scn, "HATCH_A", cbuf);
		sprintf (cbuf, "%d %0.4f", HATCHB_status, HATCHB_proc);
		oapiWriteScenario_string (scn, "HATCH_B", cbuf);
		sprintf (cbuf, "%d %0.4f", HATCHC_status, HATCHC_proc);
		oapiWriteScenario_string (scn, "HATCH_C", cbuf);
			}

void Gull::clbkPreStep (double simt, double simdt, double mjd) 
{
 ///////////////////////////////////////////////////////////////////
       // position relative to object
      VESSEL *v = oapiGetFocusInterface();// to determine which vehicle has focus  
      MinDist1 =  _V(20000,20000,20000); //Distance to trigger sound
      hbase=oapiGetVesselByName("Z'Tik-ta");// name of vessel to detect distance (shows in F3 list)
	     if (oapiIsVessel(hbase) == true) { // check if vessel exist
		  v->GetRelativePos (hbase, RelPos1);//crashes if hvessel isnt in the senario
	  		                                     //
// change numbers to positive
       if (RelPos1.x < 0.0){
 	RelPos1.x = -(RelPos1.x) ;}
       if (RelPos1.y < 0.0){
    RelPos1.y = -(RelPos1.y) ;}
       if (RelPos1.z < 0.0){ 
    RelPos1.z = -(RelPos1.z) ;} 
   






     if (MinDist1.x > RelPos1.x && MinDist1.y > RelPos1.y && MinDist1.z > RelPos1.z){

		 if (Play_Sound1 < 1){ // play once !
    
			 PlayVesselRadioExclusiveWave3(Gullwind,1,255); 
			 Play_Sound1 = 1;


		 }// play once
	 }

	
		 } }				