// ==============================================================
//                 ORBITER MODULE: Atlantis
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2004 Martin Schweiger
//                   All rights reserved
//
// W.h
// W basic specs and animations
//
// W mesh and textures by David Sundstrom
// ==============================================================

#ifndef __Cargopod_H
#define __Cargopod_H

#define STRICT
#include "orbitersdk.h"

// ==============================================================
// Some parameters and capabilities
// ==============================================================

const double Cargopod_FUELMASS = 50.0;
const double Cargopod_ISP = 6.25e4;
const double Cargopod_MAXRCSTH = 5.5e1;
const double HATCH_OPERATING_SPEED = 0.1;

// ==============================================================
// W class interface
// ==============================================================

class Cargopod: public VESSEL2 {
public:
	Cargopod (OBJHANDLE hObj, int fmodel);
	enum HATCHStatus { HATCH_UP, HATCH_DOWN, HATCH_RAISING, HATCH_LOWERING } HATCH_status;
	enum HATCHAStatus { HATCHA_UP, HATCHA_DOWN, HATCHA_RAISING, HATCHA_LOWERING } HATCHA_status;
	enum HATCHBStatus { HATCHB_UP, HATCHB_DOWN, HATCHB_RAISING, HATCHB_LOWERING } HATCHB_status;
	enum HATCHCStatus { HATCHC_UP, HATCHC_DOWN, HATCHC_RAISING, HATCHC_LOWERING } HATCHC_status;
	void DefineAnimations (void); // Useful for animation
	void RevertHATCH (void); // Operates the HATCH only
	void RevertHATCHA (void);
	void RevertHATCHB (void);
	void RevertHATCHC (void);
	void SecureCargo(void);
	void SecureCargoA(void);
	void SecureCargoB(void);
	void SecureCargoC(void);
	void SecureWaldo(void);
    void clbkLoadStateEx (FILEHANDLE scn, void *vs);//loads the status from the scenario file
	void clbkSaveState (FILEHANDLE scn);// for saving status to scenario file
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd); // Animation
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate); // Used to control events such as animation
	ATTACHMENTHANDLE sat_attach,sat_attachA,sat_attachB,sat_attachC,waldo_attach;
private:
	UINT anim_HATCH;
	double HATCH_proc;
	UINT anim_HATCHA;
	double HATCHA_proc;
	UINT anim_HATCHB;
	double HATCHB_proc;
	UINT anim_HATCHC;
	double HATCHC_proc;
};

#endif // !__Cargopod_H