
#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "ORE.h"




ORE::ORE (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)
{
	AddMesh("55Cancri//ORE");
	lnk_attach = NULL;
}

void ORE::Network()
{
	VECTOR3 grms;
	VECTOR3 cent;
	VECTOR3 pos,dir,rot;
	lnk_attach = CreateAttachment (false, _V(0,1.222,-5.052), _V(0,-1,0), _V(0,0,1), "G",true);
	Local2Global (cent, grms);
	VECTOR3 rpos;
	for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetRelativePos(GetHandle(), hV, &rpos);
			if(length(rpos) < 20) {
			VESSEL *v = oapiGetVesselInterface (hV);
	DWORD nAttach = v->AttachmentCount (true);
	for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points
	ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
	v->GetAttachmentParams (hAtt, pos, dir, rot);
		AttachChild (hV, lnk_attach, hAtt);
		return;
				}
			}
		}
}

void ORE::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main,th_hover, th_rcs[14], th_group[4];

	SetSize (6);
	SetEmptyMass (50);
	SetCW (0.09, 1.0, 1.8, 1.8);
	SetReentryTexture (NULL);
	SetWingAspect (1);
	SetWingEffectiveness (1);
	SetCrossSections (_V(5,6,5));
	SetRotDrag (_V(0.6,0.6,0.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (5e-5);
		SetBankMomentScale (5e-5);
	}
	SetPMI (_V(5,5,5));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0,0));
    SetTouchdownPoints (_V(0,-3,1.2), _V(-1.53,-3.,-4.83), _V(1.53,-3.,-4.83));

	// propellant resources
	PROPELLANT_HANDLE hpr;
	hpr = CreatePropellantResource (ZENI_FUELMASS);


	// ***************** thruster definitions *******************
     
     
	
	th_main = CreateThruster (_V(0,0,-5), _V(0,0,1), ZENI_MAXMAINTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	th_hover = CreateThruster (_V(0,-1,0), _V(0,1,0), ZENI_MAXMAINTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	CreateThrusterGroup (&th_hover, 1, THGROUP_HOVER);
	AddExhaust (th_main, .5, .1, _V(0,3.36,-.45), _V(0,0,-1));
    AddExhaust (th_main, .5, .1, _V(0,-3.36,-.45), _V(0,0,-1));
    
	

	
	th_rcs[ 0] = CreateThruster (_V( .45,3.36, 0), _V(-1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//tp
	AddExhaust (th_rcs[0], .5, .05, _V(.45,3.36,0), _V(1,0,0));
	
	th_rcs[ 1] = CreateThruster (_V(-.45,3.36,0), _V(1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//ts
	AddExhaust (th_rcs[1], .5, .05, _V(-.45,3.36,0), _V(-1,0,0));
	
	th_rcs[2] = CreateThruster (_V(0,3.36,.45), _V(0,0,-1), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//tb
	AddExhaust (th_rcs[2], .5, .05, _V(0,3.36,.45), _V(0,0,1));
	
	th_rcs[3] = CreateThruster (_V( 0,3.36,-.45), _V(0,0,1), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//tf
	AddExhaust (th_rcs[3], .5, .05, _V(0,3.36,-.45), _V(0,0,-1));
	
	th_rcs[4] = CreateThruster (_V(0,3.78,0), _V(0,-1,0), ORE_MAXRCSTH/.5, hpr, VAC_ZENI_ISP);//down
	AddExhaust (th_rcs[4], .5, .05, _V(0,3.78,0), _V(0,1,0));
	
	th_rcs[5] = CreateThruster (_V(.45,-3.36,0 ), _V(-1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//bp
	AddExhaust (th_rcs[5], .5, .05, _V(.45,-3.36,0), _V(1,0,0));
	
	th_rcs[6] = CreateThruster (_V(-.45,-3.36,0), _V(1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//bs
	AddExhaust (th_rcs[6], .5, .05, _V(-.45,-3.36,0), _V(-1,0,0));
	
	th_rcs[7] = CreateThruster (_V( 0,-3.36,.45), _V(0,0,-1), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//bb
	AddExhaust (th_rcs[7], .5, .05, _V( 0,-3.36,.45), _V(0,0,1));
	
	th_rcs[8] = CreateThruster (_V( 0,-3.36,-.45), _V(0,0,1), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//bf
	AddExhaust (th_rcs[8], .5, .05, _V( 0,-3.36,-.45), _V(0,0,-1));
	
	th_rcs[9] = CreateThruster (_V(0,-3.78,0), _V(0,1,0), ORE_MAXRCSTH/.5, hpr, VAC_ZENI_ISP);//up
	AddExhaust (th_rcs[9], .5, .05, _V(0,-3.78,0), _V(0,-1,0));
	
	th_rcs[10] = CreateThruster (_V(0,0,3.36), _V(1,0,0), ORE_MAXRCSTH/.5, hpr, VAC_ZENI_ISP);//YSfwd
	
	
	th_rcs[11] = CreateThruster (_V(0,0,-3.36), _V(-1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//ysaft
	
	
	th_rcs[12] = CreateThruster (_V(0,0,3.36), _V(-1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//ypfwd
	
	th_rcs[13] = CreateThruster (_V(0,0,-3.36), _V(1,0,0), ORE_MAXRCSTH, hpr, VAC_ZENI_ISP);//ypaft
	
	
	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[3];
	th_group[1] = th_rcs[7];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[6];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[9];
	CreateThrusterGroup (th_group, 1, THGROUP_ATT_UP);

	th_group[0] = th_rcs[4];
	CreateThrusterGroup (th_group, 1, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[12];
	th_group[1] = th_rcs[13];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[11];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[6];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);
	
    th_group[0] = th_rcs[3];
	th_group[1] = th_rcs[8];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_FORWARD);
	
	th_group[0] = th_rcs[2];
	th_group[1] = th_rcs[7];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BACK);
}

void ORE::clbkPostStep (double simt, double simdt, double mjd)
{
	
	if (GetSize() ==6.05)
	{
		Network();
		SetSize(6);
	}
	if (GetSize() ==6.06)
	{
	DetachChild (lnk_attach, 0.01);
	lnk_attach = NULL;
	SetSize(6);
	}
	
}
// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ORE (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ORE*)vessel;
}
