
#define STRICT
#define ORBITER_MODULE
#define _CRT_SECURE_NO_DEPRECATE 1
#include "orbitersdk.h"
#include "OrbiterSoundSDK3.h"
#include "Tug.h"


Tug::Tug (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)
{
	Rview		=false;
	pod_attach	=NULL;
}

void Tug::AttachPod()
{
	VECTOR3 grms;
	VECTOR3 cent;
	VECTOR3 pos,dir,rot;

	Local2Global (cent, grms);
	double ownsize = GetSize();
	VECTOR3 rpos;
	for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			double othersize = oapiGetSize(hV);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetRelativePos(GetHandle(), hV, &rpos);
			if(length(rpos) < 20) {
			VESSEL *v = oapiGetVesselInterface (hV);
	DWORD nAttach = v->AttachmentCount (true);
	for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points
	ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
	const char *id = v->GetAttachmentId (hAtt);
	if (strncmp (id, "TU", 2)) continue; // attachment point not compatible
	v->GetAttachmentParams (hAtt, pos, dir, rot);
		AttachChild (hV, pod_attach, hAtt);
		return;
				}
			}
		}
}
void Tug::RemoteView()
{

double mjd_ref=0;
char name[256];
    strcpy (name, "Earth");
    hearth=oapiGetObjectByName(name);

	VECTOR3 grms;
	VECTOR3 cent;
	

	Local2Global (cent, grms);
	double ownsize = GetSize();
	VECTOR3 rpos;
	for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			double othersize = oapiGetSize(hV);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetRelativePos(GetHandle(), hV, &rpos);
			if(length(rpos) < 500) {
				if (oapiGetSize(hV)==30){
			VESSEL *T = oapiGetVesselInterface (hV);
	hves=GetHandle();
	VESSEL *M = oapiGetVesselInterface (hves);

	
	oapiGetGlobalPos(hV,&Tglob);
	oapiGetGlobalPos(hves,&Mglob);
	M->Global2Local(Tglob,Tloc);
	M->Global2Local(Mglob,Mloc);
	Rloc=Tloc-Mloc+OFS_POD;
	M->GetRelativeVel(hV,RVel);
	Global2Local((RVel + Mglob), RLVel);
	sprintf(oapiDebugString(), "%f,&f,%f",Rloc.x,Rloc.y,Rloc.z);
			}
		}
	}
}

void Tug::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_retro, th_hover,th_rcs[14], th_group[4];

	SetSize (40);
	SetEmptyMass (40000);
	SetCW (0.09, 1.0, 1.8, 1.8);
	SetReentryTexture (NULL);
	SetWingAspect (0.7);
	SetWingEffectiveness (2.5);
	SetCrossSections (_V(300,400,145));
	SetRotDrag (_V(0.6,0.6,0.35));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (5e-5);
		SetBankMomentScale (5e-5);
	}
	SetPMI (_V(206,206,32));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-11.869,-7.027,-11.036));
	if (!pod_attach)  pod_attach = CreateAttachment (false, _V(0,0,-14.87), _V(0,0,1), _V(0,1,0), "TU", true);
	//SetDockParams (_V(0,0,-14.87), _V(0,0,1), _V(0,1,0));
	SetSurfaceFrictionCoeff (1, 1);
    EnableTransponder (true);

	// ******************** NAV radios **************************

	InitNavRadios (2);

	
	SetTouchdownPoints (_V(0,-9.1,46), _V(7.86,4.5,46), _V(-7.86,4.5,46));
	SetMeshVisibilityMode (AddMesh("55Cancri//Tug"), MESHVIS_ALWAYS);

	// propellant resources
	PROPELLANT_HANDLE hpr;
	hpr = CreatePropellantResource (ZENI_FUELMASS);
	

	
	th_main = CreateThruster (_V(0,0,-20), _V(0,0,1), ZENI_MAXMAINTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	AddExhaust (th_main, 10, 2, _V(0,4.15,-20.5), _V(0,0,-1));
    AddExhaust (th_main, 10, 2, _V(-3.54,-2,-20.5), _V(0,0,-1));
    AddExhaust (th_main, 10, 2, _V(3.6,-2,-20.5), _V(0,0,-1));
    
    th_retro = CreateThruster (_V(0,0,45.5), _V(0,0,-1), ZENI_MAXRETROTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);
	AddExhaust (th_retro, 15, 2, _V(0,9.1,46.5), _V(0,0,1));
    AddExhaust (th_retro, 15, 2, _V(-7.82,-4.5,46.5), _V(0,0,1));
    AddExhaust (th_retro, 15, 2, _V(7.87,-4.5,46.5), _V(0,0,1));

	th_hover = CreateThruster (_V(0,0,45.5), _V(0,0,-1), ZENI_MAXRETROTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_hover, 1, THGROUP_HOVER);
	AddExhaust (th_hover, 15, 2, _V(0,9.1,46.5), _V(0,0,1));
    AddExhaust (th_hover, 15, 2, _V(-7.82,-4.5,46.5), _V(0,0,1));
    AddExhaust (th_hover, 15, 2, _V(7.87,-4.5,46.5), _V(0,0,1));

	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0,1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V(1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V(1,0,0), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0,1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), AS_MAXRCSTH, hpr, VAC_ZENI_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);
	
}

// ==============================================================
// API callback interface
// ==============================================================
int Tug::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	if (KEYMOD_CONTROL (kstate)) {

		switch (key) {
		case OAPI_KEY_J: // docking
			DetachChild(pod_attach,-3);
			CreateDock(_V(0,0,-15.4),_V(0,0,1),_V(0,1,0));
			return 1;
		}
	}else{
		switch (key) {
		case OAPI_KEY_1: // landing view
			Rview=false;
			SetCameraOffset (_V(-11.869,-7.027,-11.036));
			SetCameraDefaultDirection(_V(0,0,1));
			return 1;
		case OAPI_KEY_2: // launch
			Rview=false;
			SetCameraOffset (_V(-11.869,-7.027,-14.679));
			SetCameraDefaultDirection(_V(0,0,-1));
			return 1;
		case OAPI_KEY_3: // docking
			Rview=false;
			SetCameraOffset (_V(0,0,-10.79));
			SetCameraDefaultDirection(_V(0,0,1));
			return 1;
		case OAPI_KEY_4: // docking
			Rview=true;
			return 1;
		case OAPI_KEY_G: // docking
			AttachPod();
			return 1;
		case OAPI_KEY_J: // docking
			ClearDockDefinitions();
			DetachChild(pod_attach,0);
			return 1;
		}
	}
	return 0;
}
void Tug::clbkPostStep (double simt, double simdt, double mjd)
{
	RemoteView();
	if (Rview==true){
		SetCameraOffset (Rloc);
		SetCameraDefaultDirection(_V(0,0,1));
	}


}
// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Tug (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (Tug*)vessel;
}
void Tug::clbkPostCreation (void)
{
	Calamari=ConnectToOrbiterSoundDLL3(GetHandle());

	
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\Scout\\ZENI.wav",	REPLACE_MAIN_THRUST);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB1.wav",	REPLACE_COCKPIT_AMBIENCE_1);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB2.wav",	REPLACE_COCKPIT_AMBIENCE_2);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB3.wav",	REPLACE_COCKPIT_AMBIENCE_3);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB4.wav",	REPLACE_COCKPIT_AMBIENCE_4);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB7.wav",	REPLACE_COCKPIT_AMBIENCE_7);
	ReplaceStockSound3(Calamari,"Sound\\Rho1\\landerZRG\\AMB8.wav",	REPLACE_COCKPIT_AMBIENCE_8);
	SoundOptionOnOff3(Calamari,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
}