
#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

const double ZENI_FUELMASS = 17000.0;
const double VAC_ZENI_ISP = 6.25e8;
const double NML_ZENI_ISP = 1e1;
const double ZENI_MAXMAINTH = 2e6;
const double SC_MAXRCSTH = 5e6;
const double P_NML = 1e1;


class SC: public VESSEL2 {
public:
	SC (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
};


void SC::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_rcs[14], th_group[4];

	SetSize (115);
	SetEmptyMass (1350000);
	SetCW (0.09, 1.0, 1.8, 1.8);
	SetReentryTexture (NULL);
	SetWingAspect (1);
	SetWingEffectiveness (1);
	SetCrossSections (_V(5962,6809,1804));
	SetRotDrag (_V(0.6,0.6,0.35));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (5e-5);
		SetBankMomentScale (5e-5);
	}
	SetPMI (_V(3891,3906,247));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.0,110.0));
	SetTouchdownPoints (_V(0,-20,-108), _V(-10,20,-108), _V(10,20,-108));

	// propellant resources
	PROPELLANT_HANDLE hpr;
	hpr = CreatePropellantResource (ZENI_FUELMASS);


	// ***************** thruster definitions *******************
     
     
	
	th_main = CreateThruster (_V(0,0,-40), _V(0,0,1), ZENI_MAXMAINTH, hpr, VAC_ZENI_ISP, NML_ZENI_ISP, P_NML);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	AddExhaust (th_main, 6, 1, _V(0,0,-41.8), _V(0,0,-1));
   
	

	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0,1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0,1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0,1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0,1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V(1,0,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V(1,0,0), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0,1), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), SC_MAXRCSTH, hpr, VAC_ZENI_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	// visual specs
	//AddMesh ("SC2");
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SC (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SC*)vessel;
}
