// ==============================================================
//                 ORBITER MODULE: Atlantis
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2004 Martin Schweiger
//                   All rights reserved
//
// W.h
// W basic specs and animations
//
// W mesh and textures by David Sundstrom
// ==============================================================

#ifndef __W_H
#define __W_H

#define STRICT
#include "orbitersdk.h"



// ==============================================================
// Some parameters and capabilities
// ==============================================================

const double W_FUELMASS = 50.0;
const double W_ISP = 6.25e4;
const double W_MAXMAINTH = 5.5e2;
const double W_MAXRETROTH = 5.5e2;
const double W_MAXRCSTH = 5.5e1;
const double ARM_OPERATING_SPEED = 0.25;
const VECTOR3 OFS_ZERO             = { 0.0, 0.0,  0.0  };
// ==============================================================
// W class interface
// ==============================================================

class W: public VESSEL2 {
	friend BOOL CALLBACK RMS_DlgProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
public:
	enum ARMStatus { ARM_UP, ARM_DOWN, ARM_RAISING, ARM_LOWERING } ARM_status;
	W (OBJHANDLE hVessel, int flightmodel);
	~W();
	void DefineAnimations (void); // Useful for animation
	void AddVisual (const VECTOR3 &ofs);
	void UpdateMesh ();
	void SetAnimationArm (UINT anim, double state);
	void ToggleGrapple (void);
	void ToggleArrest (void);
	void CreateLink(void);
	void KillLink(void);
	void RemoteThrust(void);
	void RemoteThrustLin(void);
	void RemoteThrustKillRot(void);
	void RemoteThrustHoldAlt(void);
	void RemoteThrustHLevel(void);
	void RemoteThrustOff(void);
	ATTACHMENTHANDLE sat_attach, rms_attach;
	VECTOR3 arm_tip[3];
	bool arm_moved;
	bool center_arm;
	bool range;
	double center_arm_t;
	VECTOR3 orbiter_ofs;
	VISHANDLE vis;
	MESHHANDLE hWaldoMesh;

    void clbkLoadStateEx (FILEHANDLE scn, void *vs);//loads the status from the scenario file
	void clbkSaveState (FILEHANDLE scn);// for saving status to scenario file
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd); // Animation
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate); // Used to control events such as animation
	void clbkAnimate (double simt);

	
private:
	bool SatGrappled() const { return GetAttachmentStatus (rms_attach) != 0; }
	
	// RMS arm animation status
	ANIMATIONCOMPONENT_HANDLE hAC_arm, hAC_sat, hAC_satref;
	MGROUP_TRANSFORM *rms_anim[6];
	UINT anim_arm_sp, anim_arm_ep,anim_arm_cp,anim_arm_cgu,anim_arm_cgl,anim_arm_at;
	double arm_sp, arm_ep, arm_cp, arm_cgu, arm_cgl,arm_at;
	
	MGROUP_TRANSFORM *sat_anim, *sat_ref;

	bool reset_mmu, reset_sat,rot_thrust,lin_thrust;
	OBJHANDLE hMMU, hSAT;
};
	
#endif // !__W_H