//############################################################################//
/*
Project Frontier
ZeRG vessel v0.2

v0.1:
Initial code

v0.1-0.2 (061101):Artlav
Cleaned the code
Remade scenario handling


Copyright 2006 Frontier team
*/
//############################################################################//
#define STRICT
#define ORBITER_MODULE

#include "ZeRG.h"      
//############################################################################//

//############################################################################//
//####################### Setting vessel parameters ##########################//
//############################################################################//
void ZERG::clbkSetClassCaps(FILEHANDLE cfg)
{
 SetSize(115);
 SetEmptyMass(13350000);
 SetCW(0.09,1.0,1.8,1.8);
 SetReentryTexture(NULL);
 SetCrossSections(_V(8352,8366,1009));
 SetRotDrag(_V(0.6,0.6,0.35));
 if(GetFlightModel() >= 1) {
  SetPitchMomentScale(5e-5);
  SetBankMomentScale(5e-5);
 }
 SetPMI(_V(4514,4514,137));
 SetTrimScale(0.05);
 SetCameraOffset(_V(0,0,115));				      
 SetTouchdownPoints(_V(0,-16.35,1.2),_V(-1.53,-16.11,-4.83),_V(1.53,-16.11,-4.83));

 dh_docks[0]=CreateDock( _V(0,0,103.06),_V(0,0,1),_V(0,1,0));
 dh_docks[1]=CreateDock(_V(0,-9.75,-71.9),_V(0,-1,0),_V(0,0,1));
 dh_docks[2]=CreateDock(_V(0,9.75,-71.9),_V(0,1,0),_V(0,0,1));

 hpr=CreatePropellantResource(ZERG_FUELMASS);		
 th_main=CreateThruster(_V(0,0,-4.5),_V(0,0,1),ZERG_MAXMAINTH,hpr,ZERG_ISP);
 CreateThrusterGroup(&th_main,1,THGROUP_MAIN);
 AddExhaust(th_main,12,1,_V(0,0,-110),_V(0,0,-1));
      
 th_rcs[ 0]=CreateThruster(_V( 1,0, 3),_V( 0, 1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 1]=CreateThruster(_V( 1,0, 3),_V( 0,-1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 2]=CreateThruster(_V(-1,0, 3),_V( 0, 1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 3]=CreateThruster(_V(-1,0, 3),_V( 0,-1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 4]=CreateThruster(_V( 1,0,-3),_V( 0, 1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 5]=CreateThruster(_V( 1,0,-3),_V( 0,-1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 6]=CreateThruster(_V(-1,0,-3),_V( 0, 1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 7]=CreateThruster(_V(-1,0,-3),_V( 0,-1,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 8]=CreateThruster(_V( 1,0, 3),_V(-1, 0,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[ 9]=CreateThruster(_V(-1,0, 3),_V( 1, 0,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[10]=CreateThruster(_V( 1,0,-3),_V(-1, 0,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[11]=CreateThruster(_V(-1,0,-3),_V( 1, 0,0),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[12]=CreateThruster(_V( 0,0,-3),_V( 0, 0,1),ZERG_MAXRCSTH,hpr,ZERG_ISP);
 th_rcs[13]=CreateThruster(_V( 0,0, 3),_V( 0,0,-1),ZERG_MAXRCSTH,hpr,ZERG_ISP);

 th_group[0]=th_rcs[0];
 th_group[1]=th_rcs[2];
 th_group[2]=th_rcs[5];
 th_group[3]=th_rcs[7];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHUP);

 th_group[0]=th_rcs[1];
 th_group[1]=th_rcs[3];
 th_group[2]=th_rcs[4];
 th_group[3]=th_rcs[6];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHDOWN);

 th_group[0]=th_rcs[0];
 th_group[1]=th_rcs[4];
 th_group[2]=th_rcs[3];
 th_group[3]=th_rcs[7];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKLEFT);

 th_group[0]=th_rcs[1];
 th_group[1]=th_rcs[5];
 th_group[2]=th_rcs[2];
 th_group[3]=th_rcs[6];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKRIGHT);

 th_group[0]=th_rcs[0];
 th_group[1]=th_rcs[4];
 th_group[2]=th_rcs[2];
 th_group[3]=th_rcs[6];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_UP);

 th_group[0]=th_rcs[1];
 th_group[1]=th_rcs[5];
 th_group[2]=th_rcs[3];
 th_group[3]=th_rcs[7];
 CreateThrusterGroup(th_group,4,THGROUP_ATT_DOWN);

 th_group[0]=th_rcs[8];
 th_group[1]=th_rcs[11];
 CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWLEFT);

 th_group[0]=th_rcs[9];
 th_group[1]=th_rcs[10];
 CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWRIGHT);

 th_group[0]=th_rcs[8];
 th_group[1]=th_rcs[10];
 CreateThrusterGroup(th_group,2,THGROUP_ATT_LEFT);

 th_group[0]=th_rcs[9];
 th_group[1]=th_rcs[11];
 CreateThrusterGroup(th_group,2,THGROUP_ATT_RIGHT);

 CreateThrusterGroup(th_rcs+12,1,THGROUP_ATT_FORWARD);
 CreateThrusterGroup(th_rcs+13,1,THGROUP_ATT_BACK);

 AddMesh("ZeRG");
}
//############################################################################//
//################################## ETC #####################################//
//############################################################################//
DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel,int flightmodel){return new ZERG(hvessel,flightmodel);}
DLLCLBK void ovcExit(VESSEL *vessel){if(vessel) delete(ZERG*)vessel;}
//############################################################################//
//################## Continuing setting vessel parameters ####################//
//############################################################################//
void ZERG::clbkPostCreation(void)
{
 osnd=ConnectToOrbiterSoundDLL3(GetHandle());
	
 ReplaceStockSound3(osnd,"Sound\\Rho1\\Scout\\ZENI.wav",    REPLACE_MAIN_THRUST);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB1.wav",REPLACE_COCKPIT_AMBIENCE_1);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB2.wav",REPLACE_COCKPIT_AMBIENCE_2);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB3.wav",REPLACE_COCKPIT_AMBIENCE_3);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB4.wav",REPLACE_COCKPIT_AMBIENCE_4);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB5.wav",REPLACE_COCKPIT_AMBIENCE_5);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB6.wav",REPLACE_COCKPIT_AMBIENCE_6);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB7.wav",REPLACE_COCKPIT_AMBIENCE_7);
 ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB8.wav",REPLACE_COCKPIT_AMBIENCE_8);
 if(cscn==1)ReplaceStockSound3(osnd,"Sound\\Rho1\\landerZRG\\AMB9.wav",REPLACE_COCKPIT_AMBIENCE_9);
 
 SoundOptionOnOff3(osnd,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	
 RequestLoadVesselWave3(osnd,1,"Sound\\Rho1\\LanderZRG\\Mareinsert.wav",RADIO_SOUND);
}
//############################################################################//
//################################# Step #####################################//
//############################################################################//
void ZERG::clbkPreStep(double simt,double simdt,double mjd) 
{
 double Altitude=0;
 OBJHANDLE ho;
 char pnam[256];

 if (cscn==1){
  
  if(scns==0){

   ho=GetSurfaceRef();
   oapiGetObjectName(ho,pnam,256);
   
   oapiGetFocusAltitude(&Altitude);
   if((Altitude<300000)&&(!_strnicmp(pnam,"MareInfinitus",13))){
    PlayVesselRadioExclusiveWave3(osnd,1,255);
    scns=1;
   }
  }
 
 }
}     
//############################################################################//
//############################ Load from scn file ############################//
//############################################################################//
void ZERG::clbkLoadStateEx(FILEHANDLE scn,void *vs)
{
 char *line;
 cscn=scns=0;

 while(oapiReadScenario_nextline(scn,line)){
  if(!_strnicmp(line,"SCENARIO",8)){
   sscanf(line+8,"%d",&cscn);
  }else if(!_strnicmp(line,"SCNSTATE",8)){
   sscanf(line+8,"%d",&scns);
  }else{
   ParseScenarioLineEx(line,vs);
  }
 }
}
//############################################################################//
//############################## Save to scn file ############################//
//############################################################################//
void ZERG::clbkSaveState(FILEHANDLE scn)
{
 char cbuf[256];
 
 VESSEL2::clbkSaveState(scn);

 sprintf(cbuf,"%d",cscn);
 oapiWriteScenario_string(scn,"SCENARIO",cbuf);
 sprintf(cbuf,"%d",scns);
 oapiWriteScenario_string(scn,"SCNSTATE",cbuf);	
}