// ======================================================================
//                     ORBITER SOFTWARE DEVELOPMENT KIT
//                  Copyright (C) 2001-2004 Martin Schweiger
//                           All rights reserved
// VesselAPI.h
// - VESSEL class interface
// - VESSEL2 class extensions
// ======================================================================

#ifndef __VESSELAPI_H
#define __VESSELAPI_H

// reference frame flags
#define FRAME_ECL 0
#define FRAME_EQU 1

class Vessel; // Orbiter internal vessel class

// ======================================================================
// class VESSEL
// This is the base class for addon vessel classes.
// ======================================================================

class DLLEXPORT VESSEL {
public:
	VESSEL (OBJHANDLE hVessel, int fmodel = 1);
	inline int Version() const { return version; }
	static OBJHANDLE Create (const char *name, const char *classname, const VESSELSTATUS &status);
	const  OBJHANDLE GetHandle (void) const;
	char   *GetName (void) const;
	char   *GetClassName (void) const;
	int    GetFlightModel (void) const;
	int    GetDamageModel (void) const;
	bool   GetEnableFocus (void) const;
	double GetSize (void) const;
	void   SetSize (double) const;
	void   SetVisibilityLimit (double vislimit, double spotlimit = -1) const;
	double GetClipRadius (void) const;
	void   SetClipRadius (double) const;
	double GetMass (void) const;
	double GetEmptyMass (void) const;
	double GetISP (void) const;
	double GetMaxThrust (ENGINETYPE eng) const;
	double GetEngineLevel (ENGINETYPE eng) const;
	double *GetMainThrustModPtr (void) const;
	int    GetAttitudeMode (void) const;
	bool   SetAttitudeMode (int mode) const;
	int    ToggleAttitudeMode (void) const;
	DWORD  GetADCtrlMode (void) const;
	void   SetADCtrlMode (DWORD mode) const;
	void   GetAttitudeRotLevel (VECTOR3 &th) const;
	void   GetAttitudeLinLevel (VECTOR3 &th) const;
	double GetManualControlLevel (THGROUP_TYPE thgt, DWORD mode = MANCTRL_ATTMODE, DWORD device = MANCTRL_ANYDEVICE) const;
	double GetCOG_elev (void) const;
	void   GetCrossSections (VECTOR3 &cs) const;
	void   GetCW (double &cw_z_pos, double &cw_z_neg, double &cw_x, double &cw_y) const;
	double GetWingAspect (void) const;
	double GetWingEffectiveness (void) const;
	void   GetRotDrag (VECTOR3&) const;
	double GetPitchMomentScale (void) const;
	double GetBankMomentScale (void) const;
	double GetTrimScale (void) const;
	void   GetPMI (VECTOR3&) const;
	void   GetCameraOffset (VECTOR3&) const;
	void   GetCameraDefaultDirection (VECTOR3&) const;
	void   GetStatus (VESSELSTATUS &status) const;
	void   GetStatusEx (void *status) const;
	DWORD  GetFlightStatus (void) const;
	const  OBJHANDLE GetGravityRef () const;
	const  OBJHANDLE GetSurfaceRef () const;
	const  OBJHANDLE GetAtmRef () const;
	double GetAtmTemperature (void) const;
	double GetAtmDensity (void) const;
	double GetAtmPressure (void) const;
	double GetDynPressure (void) const;
	double GetMachNumber (void) const;
	double GetAltitude (void) const;
	double GetAirspeed (void) const;
	bool   GetHorizonAirspeedVector (VECTOR3 &v) const;
	bool   GetShipAirspeedVector (VECTOR3 &v) const;
	double GetAOA (void) const;
	double GetSlipAngle (void) const;
	double GetPitch (void) const;
	double GetBank (void) const;
	void   GetGlobalPos (VECTOR3 &pos) const;
	void   GetGlobalVel (VECTOR3 &vel) const;
	void   GetRelativePos (OBJHANDLE hRef, VECTOR3 &pos) const;
	void   GetRelativeVel (OBJHANDLE hRef, VECTOR3 &vel) const;
	void   GetLinearMoment (VECTOR3 &F) const;
	void   GetAngularVel (VECTOR3 &avel) const;
	void   SetAngularVel (const VECTOR3 &avel) const;
	void   GetAngularAcc (VECTOR3 &aacc) const;
	void   GetAngularMoment (VECTOR3 &amom) const;
	void   GetGlobalOrientation (VECTOR3 &arot) const;
	void   SetGlobalOrientation (const VECTOR3 &arot) const;
	OBJHANDLE GetEquPos (double &longitude, double &latitude, double &radius) const;
	OBJHANDLE GetElements (ELEMENTS &el, double &mjd_ref) const;
	bool   GetElements (OBJHANDLE hRef, ELEMENTS &el, ORBITPARAM *prm = 0, double mjd_ref = 0, int frame = FRAME_ECL) const;
	bool   SetElements (OBJHANDLE hRef, const ELEMENTS &el, ORBITPARAM *prm = 0, double mjd_ref = 0, int frame = FRAME_ECL) const;
	OBJHANDLE GetArgPer (double &arg) const;
	OBJHANDLE GetSMi (double &smi) const;
	OBJHANDLE GetApDist (double &apdist) const;
	OBJHANDLE GetPeDist (double &pedist) const;
	void   SetEnableFocus (bool enable) const;
	void   SetEmptyMass (double m) const;
	bool   SetGravityGradientDamping (double damp) const;
	double GetGravityGradientDamping (void) const;
	void   SetISP (double) const;
	void   SetMaxThrust (ENGINETYPE eng, double th) const;
	void   SetEngineLevel (ENGINETYPE eng, double level) const;
	void   IncEngineLevel (ENGINETYPE eng, double dlevel) const;
	void   SetAttitudeRotLevel (const VECTOR3 &th) const;
	void   SetAttitudeRotLevel (int axis, double th) const;
	void   SetAttitudeLinLevel (const VECTOR3 &th) const;
	void   SetAttitudeLinLevel (int axis, double th) const;
	bool   ActivateNavmode (int mode);
	bool   DeactivateNavmode (int mode);
	bool   ToggleNavmode (int mode);
	bool   GetNavmodeState (int mode);
	void   SetCOG_elev (double h) const;
	void   SetCrossSections (const VECTOR3&) const;
	void   SetCW (double cw_z_pos, double cw_z_neg, double cw_x, double cw_y) const;
	void   SetWingAspect (double aspect) const;
	void   SetWingEffectiveness (double eff) const;
	void   SetRotDrag (const VECTOR3 &rd) const;
	void   SetPitchMomentScale (double scale) const;
	void   SetBankMomentScale (double scale) const;
	void   SetPMI (const VECTOR3 &pmi) const;
	void   SetAlbedoRGB (const VECTOR3 &albedo) const;
	void   SetTrimScale (double scale) const;
	void   SetCameraOffset (const VECTOR3 &co) const;
	void   SetCameraDefaultDirection (const VECTOR3 &cd) const;
	void   SetCameraDefaultDirection (const VECTOR3 &cd, double tilt) const;
	void   SetCameraRotationRange (double left, double right, double up, double down) const;
	void   SetCameraShiftRange (const VECTOR3 &fpos, const VECTOR3 &lpos, const VECTOR3 &rpos) const;
	void   SetCameraMovement (const VECTOR3 &fpos, double fphi, double ftht, const VECTOR3 &lpos, double lphi, double ltht, const VECTOR3 &rpos, double rphi, double rtht) const;
	void   SetLiftCoeffFunc (LiftCoeffFunc lcf) const;
	void   SetTouchdownPoints (const VECTOR3 &pt1, const VECTOR3 &pt2, const VECTOR3 &pt3) const;
	void   GetTouchdownPoints (VECTOR3 &pt1, VECTOR3 &pt2, VECTOR3 &pt3) const;
	void   DefSetState (const VESSELSTATUS *status) const;
	void   DefSetStateEx (const void *status) const;
	void   SaveDefaultState (FILEHANDLE scn) const;
	void   ParseScenarioLine (char *line, VESSELSTATUS *status) const;
	void   ParseScenarioLineEx (char *line, void *status) const;
	void   ClearMeshes () const;
	void   ClearMeshes (bool retain_anim) const;
	UINT   AddMesh (const char *meshname, const VECTOR3 *ofs=0) const;
	UINT   AddMesh (MESHHANDLE hMesh, const VECTOR3 *ofs=0) const;
	UINT   InsertMesh (const char *meshname, UINT idx, const VECTOR3 *ofs=0) const;
	UINT   InsertMesh (MESHHANDLE hMesh, UINT idx, const VECTOR3 *ofs=0) const;
	bool   DelMesh (UINT idx, bool retain_anim=false) const;
	bool   ShiftMesh (UINT idx, const VECTOR3 &ofs) const;
	void   ShiftMeshes (const VECTOR3 &ofs) const;
	MESHHANDLE GetMesh (VISHANDLE vis, UINT idx) const;
	void   SetMeshVisibilityMode (UINT meshidx, WORD mode) const;
	void   SetMeshVisibleInternal (UINT meshidx, bool visible) const;
	bool   MeshgroupTransform (VISHANDLE vis, const MESHGROUP_TRANSFORM &mt) const;
	void   SetExhaustScales (EXHAUSTTYPE exh, WORD id, double lscale, double wscale) const;
	void   SetReentryTexture (SURFHANDLE tex, double plimit=6e7, double lscale=1.0, double wscale=1.0) const;
	void   ShiftCentreOfMass (const VECTOR3 &shift);
	void   ShiftCG (const VECTOR3 &shift);
	void   AddForce (const VECTOR3 &F, const VECTOR3 &r) const;

	// vessel forces: vectors
	bool GetWeightVector (VECTOR3 &G) const;
	bool GetThrustVector (VECTOR3 &T) const;
	bool GetLiftVector (VECTOR3 &L) const;
	bool GetDragVector (VECTOR3 &D) const;
	bool GetForceVector (VECTOR3 &F) const;

	// vessel forces: magnitudes
	double GetLift (void) const;
	double GetDrag (void) const;

	// Animations
	void   RegisterAnimation (void) const;
	void   UnregisterAnimation (void) const;
	UINT   RegisterAnimSequence (double defmeshstate) const; // obsolete
	bool   AddAnimComp (UINT seq, ANIMCOMP *comp); // obsolete
	bool   SetAnimState (UINT seq, double state); // obsolete
	UINT   CreateAnimation (double initial_state) const;
	bool   DelAnimation (UINT anim) const;
	ANIMATIONCOMPONENT_HANDLE AddAnimationComponent (UINT anim, double state0, double state1,
		MGROUP_TRANSFORM *trans, ANIMATIONCOMPONENT_HANDLE parent = NULL) const;
	bool   DelAnimationComponent (UINT anim, ANIMATIONCOMPONENT_HANDLE hAC);
	bool   SetAnimation (UINT anim, double state) const;

	// Recording/playback functions
	bool   Recording (void) const;
	bool   Playback (void) const;
	void   RecordEvent (const char *event_type, const char *event) const;

	// Transformations
	void   GetRotationMatrix (MATRIX3 &R) const;
	void   SetRotationMatrix (const MATRIX3 &R) const;
	void   GlobalRot (const VECTOR3 &rloc, VECTOR3 &rglob) const;
	void   HorizonRot (const VECTOR3 &rloc, VECTOR3 &rhorizon) const;
	void   HorizonInvRot (const VECTOR3 &rhorizon, VECTOR3 &rloc) const;
	void   Local2Global (const VECTOR3 &local, VECTOR3 &global) const;
	void   Global2Local (const VECTOR3 &global, VECTOR3 &local) const;
	void   Local2Rel (const VECTOR3 &local, VECTOR3 &rel) const;

	// Propellant resource management
	PROPELLANT_HANDLE CreatePropellantResource (double maxmass, double mass=-1.0, double efficiency=1.0) const;
	void   DelPropellantResource (PROPELLANT_HANDLE &ph) const;
	void   ClearPropellantResources (void) const;
	PROPELLANT_HANDLE GetPropellantHandleByIndex (DWORD idx) const;
	DWORD  GetPropellantCount (void) const;
	void   SetDefaultPropellantResource (PROPELLANT_HANDLE ph) const;
	void   SetPropellantMaxMass (PROPELLANT_HANDLE ph, double maxmass) const;
	void   SetPropellantEfficiency (PROPELLANT_HANDLE ph, double efficiency) const;
	void   SetPropellantMass (PROPELLANT_HANDLE ph, double mass) const;
	double GetPropellantMaxMass (PROPELLANT_HANDLE ph) const;
	double GetPropellantEfficiency (PROPELLANT_HANDLE ph) const;
	double GetPropellantMass (PROPELLANT_HANDLE ph) const;
	double GetPropellantFlowrate (PROPELLANT_HANDLE ph) const;
	double GetTotalPropellantMass (void) const;
	double GetTotalPropellantFlowrate (void) const;
	// The following legacy functions act on the vessel's _default_ propellant resource
	double GetFuelMass (void) const;
	double GetFuelRate (void) const;
	void   SetFuelMass (double m) const;
	void   SetMaxFuelMass (double m) const;
	double GetMaxFuelMass (void) const;

	// Thruster management
	THRUSTER_HANDLE CreateThruster (const VECTOR3 &pos, const VECTOR3 &dir, double maxth0,
		PROPELLANT_HANDLE hp=NULL, double isp0=0.0, double isp_ref=0.0, double p_ref=101.4e3) const;
	bool   DelThruster (THRUSTER_HANDLE &th) const;
	void   ClearThrusterDefinitions () const;
	THRUSTER_HANDLE GetThrusterHandleByIndex (DWORD idx) const;
	DWORD  GetThrusterCount (void) const;
	void   SetThrusterRef (THRUSTER_HANDLE th, const VECTOR3 &pos) const;
	void   SetThrusterDir (THRUSTER_HANDLE th, const VECTOR3 &dir) const;
	void   SetThrusterMax0 (THRUSTER_HANDLE th, double maxth0) const;
	double GetThrusterMax0 (THRUSTER_HANDLE th) const;
	double GetThrusterMax (THRUSTER_HANDLE th) const;
	double GetThrusterMax (THRUSTER_HANDLE th, double p_ref) const;
	void   SetThrusterIsp (THRUSTER_HANDLE th, double isp) const;
	void   SetThrusterIsp (THRUSTER_HANDLE th, double isp0, double isp_ref, double p_ref=101.4e3) const;
	double GetThrusterIsp (THRUSTER_HANDLE th) const;
	double GetThrusterIsp (THRUSTER_HANDLE th, double p_ref) const;
	double GetThrusterIsp0 (THRUSTER_HANDLE th) const;
	void   SetThrusterResource (THRUSTER_HANDLE th, PROPELLANT_HANDLE ph) const;
	PROPELLANT_HANDLE GetThrusterResource (THRUSTER_HANDLE th) const;
	void   SetThrusterLevel (THRUSTER_HANDLE th, double level) const;
	void   IncThrusterLevel (THRUSTER_HANDLE th, double dlevel) const;
	void   SetThrusterLevel_SingleStep (THRUSTER_HANDLE th, double level) const;
	void   IncThrusterLevel_SingleStep (THRUSTER_HANDLE th, double dlevel) const;
	void   GetThrusterRef (THRUSTER_HANDLE th, VECTOR3 &pos) const;
	void   GetThrusterDir (THRUSTER_HANDLE th, VECTOR3 &dir) const;
	double GetThrusterLevel (THRUSTER_HANDLE th) const;
	void   GetThrusterMoment (THRUSTER_HANDLE th, VECTOR3 &F, VECTOR3 &T) const;

	// Thruster group management
	THGROUP_HANDLE CreateThrusterGroup (THRUSTER_HANDLE *th, int nth, THGROUP_TYPE thgt) const;
	bool   DelThrusterGroup (THGROUP_HANDLE &thg, THGROUP_TYPE thgt, bool delth = false) const;
	bool   DelThrusterGroup (THGROUP_TYPE thgt, bool delth = false) const;
	THGROUP_HANDLE GetThrusterGroupHandle (THGROUP_TYPE thgt) const;
	THGROUP_HANDLE GetUserThrusterGroupHandleByIndex (DWORD idx) const;
	DWORD  GetGroupThrusterCount (THGROUP_HANDLE thg) const;
	DWORD  GetGroupThrusterCount (THGROUP_TYPE thgt) const;
	THRUSTER_HANDLE GetGroupThruster (THGROUP_HANDLE thg, DWORD idx) const;
	THRUSTER_HANDLE GetGroupThruster (THGROUP_TYPE thgt, DWORD idx) const;
	DWORD  GetUserThrusterGroupCount (void) const;
	bool   ThrusterGroupDefined (THGROUP_TYPE thgt) const;
	void   SetThrusterGroupLevel (THGROUP_HANDLE thg, double level) const;
	void   SetThrusterGroupLevel (THGROUP_TYPE thgt, double level) const;
	void   IncThrusterGroupLevel (THGROUP_HANDLE thg, double dlevel) const;
	void   IncThrusterGroupLevel (THGROUP_TYPE thgt, double dlevel) const;
	void   IncThrusterGroupLevel_SingleStep (THGROUP_HANDLE thg, double dlevel) const;
	void   IncThrusterGroupLevel_SingleStep (THGROUP_TYPE thgt, double dlevel) const;
	double GetThrusterGroupLevel (THGROUP_HANDLE thg) const;
	double GetThrusterGroupLevel (THGROUP_TYPE thgt) const;

	// Exhaust render functions
	UINT   AddExhaust (THRUSTER_HANDLE th, double lscale, double wscale, SURFHANDLE tex = 0) const;
	UINT   AddExhaust (THRUSTER_HANDLE th, double lscale, double wscale, double lofs, SURFHANDLE tex = 0) const;
	UINT   AddExhaust (THRUSTER_HANDLE th, double lscale, double wscale, const VECTOR3 &pos, const VECTOR3 &dir, SURFHANDLE tex = 0) const;
	bool   DelExhaust (UINT idx) const;
	UINT   AddExhaustRef (EXHAUSTTYPE exh, VECTOR3 &pos, double lscale = -1.0, double wscale = -1.0, VECTOR3 *dir = 0) const; // obsolete
	void   DelExhaustRef (EXHAUSTTYPE exh, WORD id) const;  // obsolete
	void   ClearExhaustRefs (void) const; // obsolete
	UINT   AddAttExhaustRef (const VECTOR3 &pos, const VECTOR3 &dir, double wscale = 1.0, double lscale = 1.0) const; // obsolete
	void   AddAttExhaustMode (UINT idx, ATTITUDEMODE mode, int axis, int dir) const; // obsolete
	void   ClearAttExhaustRefs (void) const; // obsolete

	// Particle systems
	PSTREAM_HANDLE AddParticleStream (PARTICLESTREAMSPEC *pss, const VECTOR3 &pos, const VECTOR3 &dir, double *lvl) const;
	PSTREAM_HANDLE AddExhaustStream (THRUSTER_HANDLE th, PARTICLESTREAMSPEC *pss = 0) const;
	PSTREAM_HANDLE AddExhaustStream (THRUSTER_HANDLE th, const VECTOR3 &pos, PARTICLESTREAMSPEC *pss = 0) const;
	PSTREAM_HANDLE AddReentryStream (PARTICLESTREAMSPEC *pss) const;
	bool   DelExhaustStream (PSTREAM_HANDLE ch) const;

	// Airfoil and control surface functions
	void   CreateAirfoil (AIRFOIL_ORIENTATION align, const VECTOR3 &ref, AirfoilCoeffFunc cf, double c, double S, double A) const;
	AIRFOILHANDLE CreateAirfoil2 (AIRFOIL_ORIENTATION align, const VECTOR3 &ref, AirfoilCoeffFunc cf, double c, double S, double A) const;
	AIRFOILHANDLE CreateAirfoil3 (AIRFOIL_ORIENTATION align, const VECTOR3 &ref, AirfoilCoeffFuncEx cf, void *context, double c, double S, double A) const;
	void   EditAirfoil (AIRFOILHANDLE hAirfoil, DWORD flag, const VECTOR3 &ref, AirfoilCoeffFunc cf, double c, double S, double A) const;
	bool   DelAirfoil (AIRFOILHANDLE hAirfoil) const;
	void   ClearAirfoilDefinitions (void) const;
	void   CreateControlSurface (AIRCTRL_TYPE type, double area, double dCl, const VECTOR3 &ref, int axis = AIRCTRL_AXIS_AUTO, UINT anim = (UINT)-1) const;
	CTRLSURFHANDLE CreateControlSurface2 (AIRCTRL_TYPE type, double area, double dCl, const VECTOR3 &ref, int axis = AIRCTRL_AXIS_AUTO, UINT anim = (UINT)-1) const;
	bool   DelControlSurface (CTRLSURFHANDLE hCtrlSurf) const;
	void   ClearControlSurfaceDefinitions (void) const;
	void   SetControlSurfaceLevel (AIRCTRL_TYPE type, double level) const;
	double GetControlSurfaceLevel (AIRCTRL_TYPE type) const;
	void   CreateVariableDragElement (double *drag, double factor, const VECTOR3 &ref) const;
	void   ClearVariableDragElements () const;

	// Nosewheel-steering
	void   SetNosewheelSteering (bool activate) const;
	bool   GetNosewheelSteering (void) const;

	// Docking port management
	DOCKHANDLE CreateDock (const VECTOR3 &pos, const VECTOR3 &dir, const VECTOR3 &rot) const;
	bool   DelDock (DOCKHANDLE hDock) const;
	void   ClearDockDefinitions (void) const;
	void   SetDockParams (const VECTOR3 &pos, const VECTOR3 &dir, const VECTOR3 &rot) const;
	void   SetDockParams (DOCKHANDLE hDock, const VECTOR3 &pos, const VECTOR3 &dir, const VECTOR3 &rot) const;
	void   GetDockParams (DOCKHANDLE hDock, VECTOR3 &pos, VECTOR3 &dir, VECTOR3 &rot) const;
	UINT   DockCount (void) const;
	DOCKHANDLE GetDockHandle (UINT n) const;
	OBJHANDLE GetDockStatus (DOCKHANDLE hDock) const;
	int    Dock (OBJHANDLE target, UINT n, UINT tgtn, UINT mode) const;
	bool   Undock (UINT n, const OBJHANDLE exclude = 0) const;

	// Passive attachment management
	ATTACHMENTHANDLE CreateAttachment (bool toparent, const VECTOR3 &pos, const VECTOR3 &dir, const VECTOR3 &rot, const char *id, bool loose = false) const;
	void   SetAttachmentParams (ATTACHMENTHANDLE attachment, const VECTOR3 &pos, const VECTOR3 &dir, const VECTOR3 &rot) const;
	void   GetAttachmentParams (ATTACHMENTHANDLE attachment, VECTOR3 &pos, VECTOR3 &dir, VECTOR3 &rot) const;
	const char *GetAttachmentId (ATTACHMENTHANDLE attachment) const;
	OBJHANDLE GetAttachmentStatus (ATTACHMENTHANDLE attachment) const;
	DWORD  AttachmentCount (bool toparent) const;
	DWORD  GetAttachmentIndex (ATTACHMENTHANDLE attachment) const;
	ATTACHMENTHANDLE GetAttachmentHandle (bool toparent, DWORD i) const;
	bool   AttachChild (OBJHANDLE child, ATTACHMENTHANDLE attachment, ATTACHMENTHANDLE child_attachment) const;
	bool   DetachChild (ATTACHMENTHANDLE attachment, double vel = 0.0) const;

	// beacon light management
	void   AddBeacon (BEACONLIGHTSPEC *bs);
	bool   DelBeacon (BEACONLIGHTSPEC *bs);
	void   ClearBeacons ();

	// Super-structure interface
	bool   GetSuperstructureCG (VECTOR3 &cg) const;

	// Surface parameters
	void   SetSurfaceFrictionCoeff (double mu_lng, double mu_lat) const;
	void   SetMaxWheelbrakeForce (double f) const;
	void   SetWheelbrakeLevel (double level, int which = 0, bool permanent = true) const;
	double GetWheelbrakeLevel (int which) const;

	// Navigation radio interface
	void   InitNavRadios (DWORD nnav) const;
	bool   SetNavRecv (DWORD n, DWORD ch) const;
	DWORD  GetNavRecv (DWORD n) const;
	float  GetNavRecvFreq (DWORD n) const;
	bool   SetTransponderChannel (DWORD ch) const;
	bool   SetIDSChannel (DOCKHANDLE hDock, DWORD ch) const;
	void   EnableTransponder (bool enable) const;
	void   EnableIDS (DOCKHANDLE hDock, bool enable) const;
	NAVHANDLE GetTransponder (void) const;
	NAVHANDLE GetIDS (DOCKHANDLE hDock) const;
	NAVHANDLE GetNavSource (DWORD n) const;

	// Status flags
	bool   GroundContact (void) const;
	bool   OrbitStabilised (void) const;
	bool   NonsphericalGravityEnabled (void) const;
	UINT   DockingStatus (UINT port) const;

	// Scenario editor interface
	bool   GetEditorModule (char *fname) const;

protected:
	Vessel *vessel;
	short  flightmodel;
	short  version;
};

// ======================================================================
// class VESSEL2
// Extensions to the VESSEL interface. Derive your vessel classes from
// this instead of VESSEL to make use of the latest callback interface.
// ======================================================================

class DLLEXPORT VESSEL2: public VESSEL {
public:
	// Constructor
	VESSEL2 (OBJHANDLE hVessel, int fmodel = 1);

	// Callback functions
	virtual void clbkSetClassCaps (FILEHANDLE cfg);
	virtual void clbkSaveState (FILEHANDLE scn);
	virtual void clbkLoadStateEx (FILEHANDLE scn, void *status);
	virtual void clbkSetStateEx (const void *status);
	virtual void clbkPostCreation ();
	virtual void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	virtual void clbkPreStep (double simt, double simdt, double mjd);
	virtual void clbkPostStep (double simt, double simdt, double mjd);
	virtual bool clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event);
	virtual void clbkVisualCreated (VISHANDLE vis, int refcount);
	virtual void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	virtual void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
	virtual void clbkRCSMode (int mode);
	virtual void clbkADCtrlMode (DWORD mode);
	virtual void clbkHUDMode (int mode);
	virtual void clbkMFDMode (int mfd, int mode);
	virtual void clbkNavMode (int mode, bool active);
	virtual void clbkDockEvent (int dock, OBJHANDLE mate);
	virtual void clbkAnimate (double simt);
	virtual int  clbkConsumeDirectKey (char *keystate);
	virtual int  clbkConsumeBufferedKey (DWORD key, bool down, char *keystate);
	virtual bool clbkLoadGenericCockpit ();
	virtual bool clbkLoadPanel (int id);
	virtual bool clbkPanelMouseEvent (int id, int event, int mx, int my);
	virtual bool clbkPanelRedrawEvent (int id, int event, SURFHANDLE surf);
	virtual bool clbkLoadVC (int id);
	virtual bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);
	virtual bool clbkVCRedrawEvent (int id, int event, SURFHANDLE surf);
};

// ======================================================================
// class AnimState
// Auxiliary class for defining animation states
// ======================================================================

class AnimState {
public:
	enum Action {STOPPED, CLOSED, OPEN, CLOSING, OPENING} action;
	double pos;
	void Set (Action a, double p) { action = a, pos = p; }
	bool Move (double dp) {
		if (!Moving()) return false;
		if (Closing()) {
			if ((pos = max (0.0, pos-dp)) == 0.0) action = CLOSED;
		} else {
			if ((pos = min (1.0, pos+dp)) == 1.0) action = OPEN;
		}
		return true;
	}
	bool Moving() const { return action >= CLOSING; }
	bool Static() const { return action < CLOSING; }
	bool Stopped() const { return action == STOPPED; }
	bool Closed() const { return action == CLOSED; }
	bool Open() const { return action == OPEN; }
	bool Closing() const { return action == CLOSING; }
	bool Opening() const { return action == OPENING; }
	friend OAPIFUNC void WriteScenario_state (FILEHANDLE f, char *tag, const AnimState &s);
	friend OAPIFUNC void sscan_state (char *str, AnimState &s);
};

#endif // !__VESSELAPI_H
