// ======================================================================
//                     ORBITER SOFTWARE DEVELOPMENT KIT
//                  Copyright (C) 2001-2005 Martin Schweiger
//                           All rights reserved
// MFDAPI.h
// Class interfaces for MFD instruments and MFD modes
// ======================================================================

#ifndef __MFDAPI_H
#define __MFDAPI_H

#ifndef SKIP_MFD_API
#include "OrbiterAPI.h"

class Instrument;

// ======================================================================
// class MFD
// ======================================================================

class DLLEXPORT MFD {
public:
	MFD (DWORD w, DWORD h, VESSEL *vessel);
	virtual ~MFD();
	virtual void Update (HDC hDC) = 0;
	void InvalidateDisplay ();
	void InvalidateButtons ();
	void Title (HDC hDC, const char *title) const;
	HPEN SelectDefaultPen (HDC hDC, DWORD i) const;
	HFONT SelectDefaultFont (HDC hDC, DWORD i) const;
	virtual bool ConsumeKeyBuffered (DWORD) { return false; }
	virtual bool ConsumeKeyImmediate (char*) { return false; }
	virtual bool ConsumeButton (int, int) { return false; }
	virtual char *ButtonLabel (int) { return 0; }
	virtual int  ButtonMenu (const MFDBUTTONMENU**) const { return 0; }
	virtual void WriteStatus (FILEHANDLE) const {}
	virtual void ReadStatus (FILEHANDLE) {}
	virtual void StoreStatus (void) const {}
	virtual void RecallStatus (void) {}

protected:
	DWORD W, H;   // width and height of MFD display area (pixel)
	DWORD cw, ch; // character width, height of standard MFD font 0 (pixel)
	VESSEL *pV;   // pointer to vessel interface

private:
	friend class Instrument_User;  // Orbiter private class
	Instrument_User *instr;
};

// ======================================================================
// class GraphMFD
// ======================================================================

class DLLEXPORT GraphMFD: public MFD {
public:
	GraphMFD (DWORD w, DWORD h, VESSEL *vessel);
	virtual ~GraphMFD ();
	int AddGraph (void);
	void AddPlot (int g, float *absc, float *data, int ndata, int col, int *ofs = 0);
	void SetRange (int g, int axis, float rmin, float rmax);
	void SetAutoRange (int g, int axis, int p = -1);
	void SetAutoTicks (int g, int axis);
	void SetAxisTitle (int g, int axis, char *title);
	void Plot (HDC hDC, int g, int h0, int h1, const char *title = 0);
	void FindRange (float *d, int ndata, float &dmin, float &dmax) const;

protected:
	int ngraph;  // number of graphs in MFD
	struct GRAPH {
		int nplot; // number of plots in graph
		struct PLOT {
			float *absc;
			float *data;
			int ndata;
			int col;
			int *ofs;
		} *plot;
		float absc_min, absc_max;
		float data_min, data_max;
		float absc_dtick, absc_tickmin, absc_tickscale;
		float data_dtick, data_tickmin, data_tickscale;
		int absc_minortick, data_minortick;
		char absc_title[64], data_title[64];
	} *graph;
};

// ======================================================================
// class ExternMFD
// ======================================================================

class DLLEXPORT ExternMFD {
public:
	ExternMFD (const MFDSPEC &spec);
	virtual ~ExternMFD ();
	UINT Id() const;
	bool Active () const;
	OBJHANDLE GetVessel () const;
	virtual void SetVessel (OBJHANDLE hV);
	SURFHANDLE GetDisplaySurface () const;
	const char *GetButtonLabel (int bt) const;
	bool ProcessButton (int bt, int event);
	bool SendKey (DWORD key);
	bool Resize (const MFDSPEC &spec);
	bool SetMode (int mode);
	bool OpenModeHelp () const;

	// callback functions
	virtual void clbkUpdate ();
	virtual void clbkRefreshDisplay (SURFHANDLE hSurf);
	virtual void clbkRefreshButtons ();
	virtual void clbkFocusChanged (OBJHANDLE hFocus);

protected:
	OBJHANDLE hVessel; // vessel associated with the MFD
	int DW, DH;        // display width, height (pixel)
	int pmode;         // previous mode identifier
	int nbt1, nbt2;    // number of left, right buttons
	int bty0, btdy;    // geometry parameters
	int btpressed;     // currently pressed button (-1 if none)

public:
	Instrument *instr; // reserved
};

#endif // !SKIP_MFD_API
#endif // !__MFDAPI_H

